%% clear
clc;
clear;
close all;
format compact;
format short g;

%% inital stuff

params.B1=2;
params.B2=1;

model=thermalblock_model(params);

model.mu_ranges={[0.01 100] [0.01 100]};
model.RB_mu_list = {[0.1,1],[1,0.1]};
model.ynumintervals = 25;
model.xnumintervals = 25;

model_data=model.gen_model_data(model);


%% compute matrices for discrete inner product
L2 = model_data.df_info.l2_inner_product_matrix;
L2 = (L2+L2')/2;

H1 = model_data.df_info.h10_inner_product_matrix;
H1 = (H1+H1')/2;

dirichletNodes=model_data.df_info.dirichlet_gids;

H1(dirichletNodes,:)=[];
H1(:,dirichletNodes)=[];

L2(dirichletNodes,:)=[];
L2(:,dirichletNodes)=[];

model_data.l2_inner_product_matrix_reduced = L2;
model_data.h10_inner_product_matrix_reduced = H1+L2;
model_data.En = H1;

%% Save the decomposed-matrices (affine decomposition):

dofs=1:model_data.df_info.ndofs;
dofs(model_data.df_info.dirichlet_gids)=[];
model_data.dofs=dofs;
model_data.ndofs=length(dofs);

model.decomp_mode=1;
[A b]= model.operators(model,model_data);

for q=1:size(A,2)
    model_data.operators.A{q}=A{q}(dofs,dofs);
end

for q=1:size(b,2)
    model_data.operators.b{q}=b{q}(dofs);
end

%% Reference-Parameter:
mu_bar = [1;1];

model_data.innProd.refparam = mu_bar;
model_data.innProd.refalpha = getCoercivity(model, model_data, mu_bar);

model.decomp_mode=2;
model=model.set_mu(model,mu_bar);
theta=model.operators(model,model_data);
model_data.innProd.refthetas = theta(1:2);


%% Aufgabe 1: Compare coercivity constants for different inner products

% compute coercivities and inf-sup-constants
N = 40;
alpha = zeros(3,N);
mu1_list = logspace(-2,2,N);
mu2 = 1;

for n=1:numel(mu1_list)
  mu = [mu1_list(n);mu2];
  
  alpha(1,n) = getCoercivity(model,model_data, mu, model_data.h10_inner_product_matrix_reduced);
  alpha(2,n) = getCoercivity(model,model_data, mu, model_data.En);
  alpha(3,n) = getAlpha_LB(mu, model_data.innProd);
  
end

figure(1);
loglog(mu1_list',alpha(1,:)', '-x');
hold on
loglog(mu1_list',alpha(2,:)', '-o');
loglog(mu1_list',alpha(3,:)', '--');
legend('H1-Norm', 'En-Norm', 'Lower Bound');
xlabel \mu_1;
ylabel \alpha(\mu_1);
hold off


%% Set up POD

% parameter space w.r.t. \mu_1
D = [.01 100];
logD = log10(D);

% number of snapshots n_train
n_train = 101;

% parameter training set
Xi_train = [logspace(logD(1),logD(2),n_train); ...
            ones(1,n_train)];

% compute the POD basis
model_data.POD = computePODBasis(model,model_data,Xi_train);


%% Aufgabe 3: Compare error and error estimator
n_test = 40;
Xi_test = [10.^(sort(rand(1,n_test)*diff(logD)+logD(1))); ...
           ones(1,n_test)];

err_rb_POD = zeros(n_test,1);
errEst_POD = NaN*ones(n_test,1);
errEst2_POD = NaN*ones(n_test,1);

innProd = model_data.h10_inner_product_matrix_reduced;

for n=1:n_test
  mu = Xi_test(:,n);
  
  model = model.set_mu(model, mu);
  sim_data=detailed_simulation(model,model_data);
  u=sim_data.uh.dofs(model_data.dofs);
    
  [dummy,u_rb_POD] = getPODAppr(model,model_data,mu);
  
  e = u-u_rb_POD;
  err_rb_POD(n) = sqrt(e'*innProd*e);
  
  errEst_POD(n) = getErrEst(model,model_data,mu,u_rb_POD);
  errEst2_POD(n) = getErrEst2(model,model_data,mu,u_rb_POD);
end

figure(2);
e = [err_rb_POD errEst_POD errEst2_POD];
loglog(Xi_test(1,:),e,'x-')
title('Error and error estimator')
legend('Error', 'Error Bound', 'Error Bound with alpha_{LB}');
xlabel \mu_1

figure(3);
semilogx(Xi_test(1,:),errEst_POD./err_rb_POD,'x-');
title('Effectivity')
xlabel \mu_1

